﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

/*
Dayu C# Version 1.01
Replacing katakana with ASCII according to sortk.txt
Copyright 2019 Sogaya

Visual C# 2008 Express
Visual Studio 2010

Dayu assumes that the little-endian architecture underlies operating systems (Linux, Windows).
*/

namespace Dayu
{
    class Program
    {
        static void Main(string[] args)
        {
            const bool WITH_COMMENTS = true;
            const bool WITHOUT_COMMENTS = false;
            const double VERSION = 1.01;
            const int ERROR_CODE = 1;
            const string ASCII = "-a";
            const string CARRIAGE_RETURN_LINE_FEED = "-CRLF";
            const string DAYU_TEXT = "Dayu.txt";
            const string EMPTY = "";
            const string KATAKANA = "-k";
            const string LINE_FEED = "-LF";
            const string LIST_FILE = "-f";
            const string MS932 = "-m";
            const string MS932_SJIS_BIN = "MS932~Shift_JIS.bin";
            const string NO_READING = "-r-";
            const string SHIFT_JIS = "-e";
            const string SORTK_TEXT = "sortk.txt";
            const string VERSION_WITH_COMMENTS = "-VC";
            const string VERSION_WITHOUT_COMMENTS = "-V";
            const string WHICH_TABLE = "-w";

            bool carriage_return = true;
            bool document_to_be_read = true;
            bool list_to_be_read = false;
            bool noReading = false;
            bool whichTable = false;
            Editing e = new Editing();
            Guide g = new Guide();
            int exit_code = 0;
            string binaryName = MS932_SJIS_BIN;
            string documentName = EMPTY;
            string head = EMPTY;
            string list_text = EMPTY;
            string newName = DAYU_TEXT;
            string normal = "⑳"; // 8753h
            string substitute = "廿"; // 93F9h

            if (args.Length == 0) 
            {
                Standard_input i = new Standard_input();
                exit_code = i.standard_input_output(whichTable);
                System.Environment.Exit(exit_code);
            }

            foreach (string argument in args)
            {
                switch (argument)
                {
                    case CARRIAGE_RETURN_LINE_FEED:
                        carriage_return = true;
                        break;
                    case LINE_FEED:
                        carriage_return = false;
                        break;
                    case LIST_FILE:
                        list_to_be_read = true;
                        break;
                    case NO_READING:
                        noReading = true;
                        break;
                    case VERSION_WITH_COMMENTS:
                        exit_code = g.versionGuide(VERSION, WITH_COMMENTS);
                        System.Environment.Exit(exit_code);
                        break;
                    case VERSION_WITHOUT_COMMENTS:
                        exit_code = g.versionGuide(VERSION, WITHOUT_COMMENTS);
                        System.Environment.Exit(exit_code);
                        break;
                    case WHICH_TABLE:
                        whichTable = true;
                        break;
                    default:
                        head = argument.Substring(0, 2);
                        switch (head)
                        {
                            case ASCII:
                                substitute = argument.Substring(2, 1);
                                list_to_be_read = false;
                                break;
                            case SHIFT_JIS:
                                substitute = argument.Substring(2, 1);
                                list_to_be_read = false;
                                break;
                            case KATAKANA:
                                normal = argument.Substring(2, 1);
                                list_to_be_read = false;
                                break;
                            case MS932:
                                normal = argument.Substring(2, 1);
                                list_to_be_read = false;
                                break;
                            default:
                                if(list_to_be_read) {
                                    list_text = argument;
                                    list_to_be_read = false;
                                } else {
                                    if(document_to_be_read) {
                                        documentName = argument;
                                        document_to_be_read = false;
                                    } else {
                                        newName = argument;
                                    }

                                }

                                break;
                        } // switch (head)

                        break;
                } // switch (argument)

            } // foreach (string argument in args)

            if (list_text.Length == 0)
            {
                e.editRecord(normal, substitute, binaryName, whichTable);
                if (documentName.Length == 0)
                {
                    documentName = SORTK_TEXT;
                }

            }
            else
            {
                e.editManyRecords(list_text, binaryName, whichTable);
                if (documentName.Length == 0)
                {
                    documentName = list_text;
                }

            }

            if(noReading) {
                System.Environment.Exit(exit_code);
            }

            whichTable = false;

            Regulate r = new Regulate(documentName, carriage_return, whichTable);
            bool b = r.reading(newName);
            exit_code = b ? 0 : ERROR_CODE;
            System.Environment.Exit(exit_code);
        }

    }
}
