﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Dayu
{
    class Regulate
    {
        const int ASCII_SIZE = 1;
        const int KANA_HANDAKUTEN = 0xDF;
        const int KANA_KUTEN = 0xA1;
        const int KANJI_SIZE = 2;
        const int KATAKANA_SIZE = 1;

        bool carriage_return, whichTable;
        string documentName;

        private int inverted(byte[] c)
        {
            int k = (int)c[0];
            k = k << 8;
            k += c[1];
            return k;
        }

        public bool reading(string binaryName)
        {
            bool b = true;
            bool kanji = false;
            byte c;
            byte firstByte = 0x40;
            byte[] a = {0, 0};
            byte[] buffer;
            byte[] w = {0, 0};
            Encoding MS932 = Encoding.GetEncoding(932);
            int size;
            string line;

            SparseSJIS s = new SparseSJIS();
            if (! s.setting(whichTable) )
            {
                Console.Error.WriteLine("s->setting(whichTable)");
                return b;
            }

            if ( File.Exists(binaryName) )
            {
                File.Delete(binaryName);
            }

            try
            {
                Stream binaryFile = File.OpenWrite(binaryName);
                StreamReader sjisFile = new StreamReader(documentName, MS932);

                for (; ; )
                {
                    line = sjisFile.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    buffer = MS932.GetBytes(line);
                    size = MS932.GetByteCount(line);
                    for (int i = 0; i < size; ++i)
                    {
                        c = buffer[i];

                        if (kanji)
                        {
                            a[0] = firstByte;
                            a[1] = buffer[i];
                            w[0] = s.working( inverted(a), 0);
                            w[1] = s.working( inverted(a), 1);
                            binaryFile.Write(w, 0, KANJI_SIZE);
                            kanji = false;
                            continue;
                        }

                        if (0x81 <= c && c<= 0xFC)
                        {
                            if (KANA_KUTEN <= c && c <= KANA_HANDAKUTEN)
                            {
                                w[0] = s.working(c, 0);
                                w[1] = 0;
                                binaryFile.Write(w, 0, KATAKANA_SIZE);
                                kanji = false;
                            }
                            else
                            {
                                firstByte = c;
                                kanji = true;
                            }

                        }
                        else
                        {
                            w[0] = s.working(c, 0);
                            w[1] = 0;
                            binaryFile.Write(w, 0, ASCII_SIZE);
                            kanji = false;
                        }

                    } // for (int i = 0; i < size; ++i)

                    if(carriage_return)
                    {
                        w[0] = 0x0D;
                        w[1] = 0x0A;
                        binaryFile.Write(w, 0, 2);
                    }
                    else
                    {
                        w[0] = 0x0A;
                        w[1] = 0;
                        binaryFile.Write(w, 0, 1);
                    }

                } //for (; ; )

                binaryFile.Close();
                sjisFile.Close();
            }
            catch (IOException e15)
            {
                Console.Error.WriteLine(documentName);
                Error.handler(e15.ToString());
                b = false;
            }

            return b;
        }

        public Regulate(string d, bool c, bool w)
        {
            documentName = d;
            carriage_return = c;
            whichTable = w;
        }

    }
}
