﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;

namespace Dayu
{
    class SparseSJIS
    {
        const byte ERROR_CODE = 0xFF;
        const int DOUBLE_KANJI = 4; // MS932_KANJI_SIZE + SJIS_KANJI_SIZE
        const int SJIS_KANJI_SIZE = 2;
        const int SJIS_KANJI_SIZE_1 = 3;
        const int FIRST_KANJI = 159;
        const int RECORD_COUNT = 7882;
        const int MS932_KANJI_SIZE = 2;
        const int YEN_SIZE = 1;
        const string C_MS932_SJIS_BIN = "C:\\sjisEUCjpCS\\MS932~Shift_JIS.bin";
        const string MS932_SJIS_BIN = "MS932~Shift_JIS.bin";
        const string YEN = "\\";

        byte[] sparse;
        int size;

        private int inverted(byte[] c)
        {
            int k = (int)c[0];
            k = k << 8;
            k += c[1];
            return k;
        }

        private string locateBinaryAssembly()
        {
            Assembly Dayu = Assembly.GetExecutingAssembly();
            int q;
            string binaryName;
            string path = Dayu.Location;

            q = path.LastIndexOf(YEN);
            q += YEN_SIZE;
            binaryName = path.Substring(0, q) + MS932_SJIS_BIN;
            return binaryName;
        }

        private void populated(int index, int place, byte c)
        {
            try
            {
                int k = index * SJIS_KANJI_SIZE_1 + place;
                sparse[k] = c;
            }
            catch (IndexOutOfRangeException e16)
            {
                Error.handler(e16.ToString());
            }

        }

        public bool setting(bool whichTable)
        {
            byte[] c = {0, 0, 0, 0};
            CommandPath comath = new CommandPath();
            int index;
            string included = comath.pathName(MS932_SJIS_BIN);
            string table = C_MS932_SJIS_BIN;
            string[] MS932_SJIS_TABLES = {"", "", "", ""};

            MS932_SJIS_TABLES[0] = C_MS932_SJIS_BIN; // directory where SjisEuc was installed.
            MS932_SJIS_TABLES[1] = MS932_SJIS_BIN; // current directory
            MS932_SJIS_TABLES[2] = included; // directory included in Path
            MS932_SJIS_TABLES[3] = locateBinaryAssembly();

            for (int i = 0; i <= 3; ++i)
            {
                try
                {
                    Stream testFile = File.OpenRead(MS932_SJIS_TABLES[i]);
                    table = MS932_SJIS_TABLES[i];
                    testFile.Close();
                    if (whichTable)
                    {
                        Console.WriteLine( table + " is primarily accessible to Dayu." );
                    }

                    break;
                }
                catch (IOException e17)
                {
                    Error.handler(e17.ToString());
                    if(i == 2) {
                        Console.Error.WriteLine(MS932_SJIS_BIN);
                        return false;
                    }

                }

            }

            Stream binaryFile = File.OpenRead(table);
            for(int i = 1; i <= RECORD_COUNT; ++i)
            {
                binaryFile.Read(c, 0, DOUBLE_KANJI);
                if (i < FIRST_KANJI)
                {
                    index = (int)c[0];
                    populated(index, 0, c[MS932_KANJI_SIZE]);
                }
                else
                {
                    populated( inverted(c), 0, c[MS932_KANJI_SIZE]);
                    populated( inverted(c), 1, c[ MS932_KANJI_SIZE + 1 ]);
                }
 
            }

            binaryFile.Close();
            return true;
        }

        public void clear()
        {
            for(int i = 0; i < sparse.Length; ++i)
            {
                sparse[i] = 0;
            }

        }

        public SparseSJIS()
        {
            size = 0xFC4C * SJIS_KANJI_SIZE_1;
            sparse = new byte[size];
        }

        public byte working(int index, int place)
        {
            int k = index * SJIS_KANJI_SIZE_1 + place;
            byte r = 0;

            try
            {
                r = sparse[k];
            }
            catch (IndexOutOfRangeException e18)
            {
                Error.handler(e18.ToString());
                Console.WriteLine( string.Format("{0:X4}", k) );
                r = ERROR_CODE;
            }

            return r;
        }

    }
}
