﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;

namespace Diyo
{
    class Editing
    {
        const bool PLEASE_CHECK = true;
        const int BLOCK_SIZE = 1024;
        const int DATA_ARRAY_LIMIT = 906;
        const int DOUBLE_KANJI = 4; // SJIS_KANJI_SIZE + EUC_KANJI_SIZE
        const int ERROR_CODE = 1;
        const int EUC_KANJI_SIZE = 2;
        const int FIRST_KANJI = 159;
        const int KANA_KUTEN = 161;
        const int KANA_HANDAKUTEN = 223;
        const int RECORD_COUNT = 7882;
        const int SJIS_KANJI_SIZE  = 2;
        const int YEN_SIZE = 1;
        const string C_SJIS_EUC_BIN = "C:\\sjisEUCjpCS\\MS932~EUC-JP.bin";
        const string TEMPORARY_BINARY = "binary.tmp";
        const string RIGHT_ARROW = " ⇒ "; // UTF-8
        const string SJIS_EUC_BIN = "MS932~EUC-JP.bin";
        const string YEN = "\\";

        NormalSubstitute[] data = new NormalSubstitute[DATA_ARRAY_LIMIT];

        private bool renameBinaryFile(string binaryName)
        {
            bool b = true;
            byte[] c = new byte[BLOCK_SIZE];
            int size;

            try
            {
                File.Delete(binaryName);
                Stream editedFile = File.OpenRead(TEMPORARY_BINARY);
                Stream finalFile = File.OpenWrite(binaryName);

                for(; ; ) {
                    size = editedFile.Read(c, 0, BLOCK_SIZE);
                    if (size > 0)
                    {
                        finalFile.Write(c, 0, size);
                    }
                    else
                    {
                        break;
                    }

                }

                editedFile.Close();
                finalFile.Close();
                File.Delete(TEMPORARY_BINARY);
            }
            catch (IOException e20)
            {
                Console.Error.WriteLine(TEMPORARY_BINARY);
                Error.handler(e20.ToString());
                b = false;
            }

            return b;
        }

        private int inverted(byte[] c)
        {
            int k = (int)c[0];
            k = k << 8;
            k += c[1];
            return k;
        }

        private int kanjiCode(string line)
        {
            bool kanji = false;
            Encoding MS932 = Encoding.GetEncoding(932);
            byte[] buffer = MS932.GetBytes(line);
            byte[] c = {0, 0, 0, 0};
            int before = 0;
            int k = 0;
            int n = 0;
            int size = MS932.GetByteCount(line);

            for (int i = 0; i < size; ++i)
            {
                k = (int)buffer[i];

                if (kanji)
                {
                    c[0] = buffer[before];
                    c[1] = buffer[i];
                    n = inverted(c);
                    break;
                }

                if (0x81 <= k && k <= 0xFC)
                {
                    if (KANA_KUTEN <= k && k <= KANA_HANDAKUTEN)
                    {
                        n = k;
                        break;
                    }
                    else
                    {
                        before = i;
                        kanji = true;
                    }

                }
                else
                {
                    n = k;
                    break;
                }

            }

            return n;
        }

        private string locateBinaryAssembly()
        {
            Assembly diyo = Assembly.GetExecutingAssembly();
            int q;
            string binaryName;
            string path = diyo.Location;

            q = path.LastIndexOf(YEN);
            q += YEN_SIZE;
            binaryName = path.Substring(0, q) + SJIS_EUC_BIN;
            return binaryName;
        }

        public bool editRecord(string normal, string substitute, string binaryName, bool whichTable)
        {
            byte[] c = {0, 0, 0, 0};
            byte[] os = {0, 0, 0, 0};
            CommandPath comath = new CommandPath();
            int from, into, k;
            string included = comath.pathName(SJIS_EUC_BIN);
            string table = C_SJIS_EUC_BIN;
            string[] SJIS_EUC_TABLES = {"", "", "", ""};

            SparseEUC s = new SparseEUC();
            if (!s.setting(whichTable))
            {
                Console.Error.WriteLine("s->setting(whichTable)");
                return false;
            }

            SJIS_EUC_TABLES[0] = C_SJIS_EUC_BIN; // directory where SjisEuc was installed.
            SJIS_EUC_TABLES[1] = SJIS_EUC_BIN; // current directory
            SJIS_EUC_TABLES[2] = included; // directory included in Path
            SJIS_EUC_TABLES[3] = locateBinaryAssembly();

            for (int i = 0; i <= 3; ++i)
            {
                try
                {
                    Stream testFile = File.OpenRead(SJIS_EUC_TABLES[i]);
                    table = SJIS_EUC_TABLES[i];
                    testFile.Close();
                    break;
                }
                catch (IOException e21)
                {
                    Error.handler(e21.ToString());
                    if (i == 2)
                    {
                        Console.Error.WriteLine(SJIS_EUC_BIN);
                        return false;
                    }

                }

            }

            from = kanjiCode(normal);
            Console.Write( string.Format("{0:X4}", from) );
            Console.Write(RIGHT_ARROW);

            into = kanjiCode(substitute);
            Console.WriteLine( string.Format("{0:X4}", into) );

            try
            {
                File.Delete(TEMPORARY_BINARY);
            }
            catch (IOException e74)
            {
                Error.handler(e74.ToString());
            }

            Stream binaryFile = File.OpenRead(table);
            Stream temporaryFile = File.OpenWrite(TEMPORARY_BINARY);
            for(int i = 1; i <= RECORD_COUNT; ++i)
            {
                binaryFile.Read(c, 0, DOUBLE_KANJI);
                if (i < FIRST_KANJI)
                {
                    k = (int)c[0];
                }
                else
                {
                    k = inverted(c);
                }

                os[0] = c[0];
                os[1] = c[1];

                if (k == from)
                {
                    os[2] = s.working(into, 0);
                    os[3] = s.working(into, 1);
                }
                else
                {
                    os[2] = c[SJIS_KANJI_SIZE];
                    os[3] = c[ SJIS_KANJI_SIZE + 1 ];
                }

                temporaryFile.Write(os, 0, DOUBLE_KANJI);
            } // for(int i = 1; i <= RECORD_COUNT; ++i)

            binaryFile.Close();
            temporaryFile.Close();
            renameBinaryFile(binaryName);
            return true;
        }

        public bool editManyRecords(string text, string binaryName, bool whichTable)
        {
            bool b = true;
            bool found = false;
            byte[] c = {0, 0, 0, 0};
            byte[] os = {0, 0, 0, 0};
            CommandPath comath = new CommandPath();
            int from, k;
            int into = 0x8750;
            int used = 0;
            string included = comath.pathName(SJIS_EUC_BIN);
            string line, normal, substitute;
            string table = C_SJIS_EUC_BIN;
            string[] SJIS_EUC_TABLES = {"", "", "", ""};

            try
            {
                StreamReader listFile = new StreamReader(text, Encoding.Default);
                for (int m = 0; m < DATA_ARRAY_LIMIT; ++m)
                {
                    line = listFile.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    normal = line.Substring(0, 1);
                    substitute = line.Substring(1, 1);
                    data[m] = new NormalSubstitute(normal, substitute, PLEASE_CHECK);
                    used = m;
                }

                listFile.Close();

                SparseEUC s = new SparseEUC();
                if (!s.setting(whichTable))
                {
                    Console.Error.WriteLine("s->setting(whichTable)");
                    return false;
                }

                SJIS_EUC_TABLES[0] = C_SJIS_EUC_BIN; // directory where SjisEuc was installed.
                SJIS_EUC_TABLES[1] = SJIS_EUC_BIN; // current directory
                SJIS_EUC_TABLES[2] = included; // directory included in Path
                SJIS_EUC_TABLES[3] = locateBinaryAssembly();

                for (int i = 0; i <= 3; ++i)
                {
                    try
                    {
                        Stream testFile = File.OpenRead(SJIS_EUC_TABLES[i]);
                        table = SJIS_EUC_TABLES[i];
                        testFile.Close();
                        break;
                    }
                    catch (IOException e22)
                    {
                        Error.handler(e22.ToString());
                        if (i == 2)
                        {
                            Console.Error.WriteLine(SJIS_EUC_BIN);
                            return false;
                        }

                    }

                }

                File.Delete(TEMPORARY_BINARY);
                Stream binaryFile = File.OpenRead(table);
                Stream temporaryFile = File.OpenWrite(TEMPORARY_BINARY);
                for (int i = 1; i <= RECORD_COUNT; ++i)
                {
                    binaryFile.Read(c, 0, DOUBLE_KANJI);
                    if (i < FIRST_KANJI)
                    {
                        k = (int)c[0];
                    }
                    else
                    {
                        k = inverted(c);
                    }

                    os[0] = c[0];
                    os[1] = c[1];

                    for (int m = 0; m <= used; ++m)
                    {
                        if (data[m].pleaseCheck())
                        {
                            from = kanjiCode(data[m].normalCharacter());
                            if (k == from)
                            {
                                into = kanjiCode(data[m].substituteCharacter());
                                found = true;
                                data[m].sure();
                                break;
                            }
                            else
                            {
                                found = false;
                            }

                        }
                        else
                        {
                            found = false;
                        }

                    }

                    if (found)
                    {
                        os[2] = s.working(into, 0);
                        os[3] = s.working(into, 1);
                    }
                    else
                    {
                        os[2] = c[SJIS_KANJI_SIZE];
                        os[3] = c[SJIS_KANJI_SIZE + 1];
                    }

                    temporaryFile.Write(os, 0, DOUBLE_KANJI);
                } // for (int i = 1; i <= RECORD_COUNT; ++i)

                binaryFile.Close();
                temporaryFile.Close();
                renameBinaryFile(binaryName);
            }
            catch (IOException e23)
            {
                Error.handler(e23.ToString());
                Console.Error.WriteLine(text);
                b = false;
            }

            return b;
        }

    }
}
