﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Hanzen
{
    class HankakuZenkaku
    {
        const int BLOCK_SIZE = 1024;
        const int ERROR_CODE = 1;
        const int HA_HI_HU_HE_HO = 15;
        const int KANA_DAKUTEN = 0xDE;
        const int KANA_HANDAKUTEN = 0xDF;
        const int KANA_KUTEN = 0xA1;
        const int ZENKAKU_SIZE = 2;
        const long FILE_ERROR = -1L;
        const string TEMPORARY_BINARY = "binary.tmp";

        byte[] buffer;
        byte[] os;
        int bufferSize;
        string fileName;

        private bool renameBinaryFile(string binaryName)
        {
            bool b = true;
            byte[] c = new byte[BLOCK_SIZE];
            int size;

            try
            {
                File.Delete(binaryName);
                Stream editedFile = File.OpenRead(TEMPORARY_BINARY);
                Stream finalFile = File.OpenWrite(binaryName);

                for(; ; ) {
                    size = editedFile.Read(c, 0, BLOCK_SIZE);
                    if (size > 0)
                    {
                        finalFile.Write(c, 0, size);
                    }
                    else
                    {
                        break;
                    }

                }

                editedFile.Close();
                finalFile.Close();
                File.Delete(TEMPORARY_BINARY);
            }
            catch (IOException e30)
            {
                Console.Error.WriteLine(TEMPORARY_BINARY);
                Error.handler(e30.ToString());
                b = false;
            }

            return b;
        }

        private bool seion(int beginning, byte first_byte, byte second_byte)
        {
            bool b = false;
            byte[] d = {0, 0};
            int[] ka_sa_ta_ha = {0x834A, 0x834C, 0x834E, 0x8350, 0x8352, 0x8354, 0x8356, 0x8358, 0x835A, 0x835C, 0x835E, 0x8360, 0x8363, 0x8365, 0x8367, 0x836E, 0x8371, 0x8374, 0x8377, 0x837A};

            d[0] = first_byte;
            d[1] = second_byte;
            int test = inverted(d);

            for (int i = beginning; i < ka_sa_ta_ha.Length; ++i)
            {
                if (ka_sa_ta_ha[i] == test)
                {
                    b = true;
                    break;
                }

            }

            return b;
        }

        private int inverted(byte[] c)
        {
            int k = (int)c[0];
            k = k << 8;
            k += c[1];
            return k;
        }

        public int reading(string binaryName, bool whichTable)
        {
            bool kanji = false;
            byte firstByte = 0x40;
            int c = 0x40;
            int exit_code = 0;
            int k = 0;
            int size = 0;
            int previous = 0xB6;

            SparseKana s = new SparseKana();
            if (! s.setting(whichTable) )
            {
                Console.Error.WriteLine("s->setting(whichTable)");
                return ERROR_CODE;
            }

            try
            {
                File.Delete(TEMPORARY_BINARY);
            }
            catch (IOException e75)
            {
                Error.handler(e75.ToString());
            }

            try
            {
                Stream binaryFile = File.OpenRead(fileName);
                Stream temporaryFile = File.OpenWrite(TEMPORARY_BINARY);
                size = binaryFile.Read(buffer, 0, bufferSize);

                for (int i = 0; i < size; ++i)
                {
                    c = (int)buffer[i];

                    if (kanji)
                    {
                        os[k] = firstByte;
                        os[k + 1] = buffer[i];
                        k += ZENKAKU_SIZE;
                        kanji = false;
                        continue;
                    }

                    if (0x81 <= c && c <= 0xFC)
                    {
                        if (KANA_KUTEN <= c && c < KANA_DAKUTEN)
                        {
                            os[k] = s.working(c, 0);
                            os[k + 1] = s.working(c, 1);
                            k += ZENKAKU_SIZE;
                            previous = c;
                        }
                        else if (c == KANA_DAKUTEN)
                        {
                            // combining character
                            k -= ZENKAKU_SIZE;

                            if ( seion(0, os[k], os[k + 1]) )
                            {
                                os[k] = s.dakuten(previous, 0);
                                os[k + 1] = s.dakuten(previous, 1);
                                k += ZENKAKU_SIZE;
                            }
                            else
                            {
                                k += ZENKAKU_SIZE;
                            }

                        }
                        else if (c == KANA_HANDAKUTEN)
                        {
                            // combining character
                            k -= ZENKAKU_SIZE;

                            if ( seion(HA_HI_HU_HE_HO, os[k], os[k + 1]) )
                            {
                                os[k] = s.handakuten(previous, 0);
                                os[k + 1] = s.handakuten(previous, 1);
                                k += ZENKAKU_SIZE;
                            }
                            else
                            {
                                k += ZENKAKU_SIZE;
                            }

                        }
                        else
                        {
                            firstByte = buffer[i];
                            kanji = true;
                        }

                    }
                    else
                    {
                        os[k++] = buffer[i];
                    }

                } // for (int i = 1; i < size; ++i)

                temporaryFile.Write(os, 0, k);
                binaryFile.Close();
                temporaryFile.Close();
                renameBinaryFile(binaryName);
            }
            catch (IOException e31)
            {
                Error.handler(e31.ToString());
                Console.Error.WriteLine(fileName);
                exit_code = ERROR_CODE;
            }

            return exit_code;
        }

        public long fileSize()
        {
            long size = FILE_ERROR;

            try
            {
                FileStream f = new FileStream(fileName, FileMode.Open, FileAccess.Read);
                size = (long) f.Length;
                f.Close();
            }
            catch (IOException e32)
            {
                Console.Error.WriteLine(fileName);
                Error.handler(e32.ToString());
            }

            return size;
        }

        public HankakuZenkaku(string f)
        {
            fileName = f;
            bufferSize = (int)fileSize();
            buffer = new byte[bufferSize];
            os = new byte[ bufferSize * 2 ];
        }

    }
}
