﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;

namespace Hanzen
{
    class SparseKana
    {
        const bool DAKUTEN = false;
        const bool HANDAKUTEN = true;
        const byte ERROR_CODE = 0xFF;
        const int DAKUTEN_OFFSET = 0x100;
        const int FIRST_DAKUTEN = 62;
        const int FIRST_HANDAKUTEN = 82;
        const int HANDAKUTEN_OFFSET = 0x200;
        const int HANKAKU_ZENKAKU = 3; // KATAKANA_SIZE + ZENKAKU_SIZE
        const int KATAKANA_SIZE = 1;
        const int RECORD_COUNT = 86;
        const int YEN_SIZE = 1;
        const int ZENKAKU_SIZE = 2;
        const int ZENKAKU_SIZE_1 = 3;
        const string C_HANZEN_TEXT = "C:\\sjisEUCjpCS\\Hankaku~Zenkaku_katakana.bin";
        const string HANZEN_TEXT = "Hankaku~Zenkaku_katakana.bin";
        const string YEN = "\\";

        byte[] sparse;
        int size;

        private int moved(byte c, bool handakuten)
        {
            int offset = handakuten ? HANDAKUTEN_OFFSET : DAKUTEN_OFFSET;
            int index = (int)c + offset;
            return index;
        }

        private string locateBinaryAssembly()
        {
            Assembly hanzen = Assembly.GetExecutingAssembly();
            int q;
            string binaryName;
            string path = hanzen.Location;

            q = path.LastIndexOf(YEN);
            q += YEN_SIZE;
            binaryName = path.Substring(0, q) + HANZEN_TEXT;
            return binaryName;
        }

        private void populated(int index, int place, byte c)
        {
            try
            {
                int k = index * ZENKAKU_SIZE_1 + place;
                sparse[k] = c;
            }
            catch (IndexOutOfRangeException e33)
            {
                Error.handler(e33.ToString());
            }

        }

        public bool setting(bool whichTable)
        {
            byte[] c = {0, 0, 0, 0};
            CommandPath comath = new CommandPath();
            int index;
            string included = comath.pathName(HANZEN_TEXT);
            string table = C_HANZEN_TEXT;
            string[] HANKAKU_ZENKAKU_TABLES = {"", "", "", ""};

            HANKAKU_ZENKAKU_TABLES[0] = C_HANZEN_TEXT; // directory where SjisEuc was installed.
            HANKAKU_ZENKAKU_TABLES[1] = HANZEN_TEXT; // current directory
            HANKAKU_ZENKAKU_TABLES[2] = included; // directory included in Path
            HANKAKU_ZENKAKU_TABLES[3] = locateBinaryAssembly();

            for (int i = 0; i <= 3; ++i)
            {
                try
                {
                    Stream testFile = File.OpenRead(HANKAKU_ZENKAKU_TABLES[i]);
                    table = HANKAKU_ZENKAKU_TABLES[i];
                    testFile.Close();
                    if (whichTable)
                    {
                        Console.WriteLine( table + " is primarily accessible to Hanzen." );
                    }

                    break;
                }
                catch (IOException e34)
                {
                    Error.handler(e34.ToString());
                    if(i == 2) {
                        Console.Error.WriteLine(HANZEN_TEXT);
                        return false;
                    }

                }

            }

            Stream binaryFile = File.OpenRead(table);
            for(int i = 1; i <= RECORD_COUNT; ++i)
            {
                binaryFile.Read(c, 0, HANKAKU_ZENKAKU);
                if(i < FIRST_DAKUTEN)
                {
                    index = (int)c[0];
                    populated(index, 0, c[KATAKANA_SIZE]);
                    populated(index, 1, c[ KATAKANA_SIZE + 1 ]);
                }
                else if (i >= FIRST_HANDAKUTEN)
                {
                    populated( moved(c[0], HANDAKUTEN), 0, c[KATAKANA_SIZE]);
                    populated( moved(c[0], HANDAKUTEN), 1, c[ KATAKANA_SIZE + 1 ]);
                }
                else
                {
                    populated( moved(c[0], DAKUTEN), 0, c[KATAKANA_SIZE]);
                    populated( moved(c[0], DAKUTEN), 1, c[ KATAKANA_SIZE + 1 ]);
                }

            }

            binaryFile.Close();
            return true;
        }

        public void clear()
        {
            for(int i = 0; i < sparse.Length; ++i)
            {
                sparse[i] = 0;
            }

        }

        public SparseKana()
        {
            size = 0x02E0 * ZENKAKU_SIZE_1;
            sparse = new byte[size];
        }

        public byte dakuten(int index, int place)
        {
            int k = ( index + DAKUTEN_OFFSET) * ZENKAKU_SIZE_1 + place;
            byte r = 0;

            try
            {
                r = sparse[k];
            }
            catch (IndexOutOfRangeException e35)
            {
                Error.handler(e35.ToString());
                Console.WriteLine( string.Format("{0:X4}", k) );
                r = ERROR_CODE;
            }

            return r;
        }

        public byte handakuten(int index, int place)
        {
            int k = ( index + HANDAKUTEN_OFFSET) * ZENKAKU_SIZE_1 + place;
            byte r = 0;

            try
            {
                r = sparse[k];
            }
            catch (IndexOutOfRangeException e36)
            {
                Error.handler(e36.ToString());
                Console.WriteLine( string.Format("{0:X4}", k) );
                r = ERROR_CODE;
            }

            return r;
        }

        public byte working(int index, int place)
        {
            int k = index * ZENKAKU_SIZE_1 + place;
            byte r = 0;

            try
            {
                r = sparse[k];
            }
            catch (IndexOutOfRangeException e37)
            {
                Error.handler(e37.ToString());
                Console.WriteLine( string.Format("{0:X4}", k) );
                r = ERROR_CODE;
            }

            return r;
        }

    }
}
