﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Hanzen
{
    class Standard_input
    {
        const int ASCII_SIZE = 1;
        const int ERROR_CODE = 1;
        const int KANA_DAKUTEN = 0xDE;
        const int KANA_HANDAKUTEN = 0xDF;
        const int KANA_KUTEN = 0xA1;
        const int KANJI_SIZE = 2;
        const int KATAKANA_SIZE = 1;
        const int NEW_LINE_SIZE = 2;
        const int ZENKAKU_SIZE = 2;
        const string NEW_LINE = "\x0D\x0A";

        public int standard_input_output(bool whichTable)
        {
            bool kanji = false;
            byte firstByte = 0x40;
            byte[] a = {0, 0};
            byte[] buffer;
            byte[] os;
            byte[] w = {0, 0};
            Encoding EUC_JP = Encoding.GetEncoding("euc-jp");
            Encoding MS932 = Encoding.GetEncoding(932);
            int c = 0x40;
            int k, size, os_size;
            int exit_code = 0;
            int previous = 0xB6;
            Stream standard_output = Console.OpenStandardOutput();
            string line;

            SparseKana s = new SparseKana();
            if (! s.setting(whichTable) )
            {
                Console.Error.WriteLine("s->setting(whichTable)");
                return ERROR_CODE;
            }

            for (; ; )
            {
                line = Console.ReadLine();
                if (line == null)
                {
                    break;
                }

                buffer = MS932.GetBytes(line);
                size = MS932.GetByteCount(line);
                os_size = size * 2 + NEW_LINE_SIZE;
                os = new byte[os_size];
                k = 0;
                for (int i = 0; i < size; ++i)
                {
                    c = (int)buffer[i];

                    if (kanji)
                    {
                        os[k] = firstByte;
                        os[k + 1] = buffer[i];
                        k += ZENKAKU_SIZE;
                        kanji = false;
                        continue;
                    }

                    if (0x81 <= c && c <= 0xFC)
                    {
                        if (KANA_KUTEN <= c && c < KANA_DAKUTEN)
                        {
                            os[k] = s.working(c, 0);
                            os[k + 1] = s.working(c, 1);
                            k += ZENKAKU_SIZE;
                            previous = c;
                        }
                        else if (c == KANA_DAKUTEN)
                        {
                            // combining character
                            k -= ZENKAKU_SIZE;

                            os[k] = s.dakuten(previous, 0);
                            os[k + 1] = s.dakuten(previous, 1);
                            k += ZENKAKU_SIZE;
                        }
                        else if (c == KANA_HANDAKUTEN)
                        {
                            // combining character
                            k -= ZENKAKU_SIZE;

                            os[k] = s.handakuten(previous, 0);
                            os[k + 1] = s.handakuten(previous, 1);
                            k += ZENKAKU_SIZE;
                        }
                        else
                        {
                            firstByte = buffer[i];
                            kanji = true;
                        }

                    }
                    else
                    {
                        os[k++] = buffer[i];
                    }

                } // for (int i = 1; i < size; ++i)

                os[k++] = 0x0D;
                os[k++] = 0x0A;
                standard_output.Write(os, 0, k);
            }
            return exit_code;
        }

        public Standard_input()
        {
        }

    }
}
