﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Narfi
{
    class Line
    {
        const int NOT_FOUND = -1;
        const int SEPARATOR_SIZE = 1;
        const long FILE_ERROR = -1L;
        const string EMPTY = "";

        bool carriageReturn, measure;
        byte[] buffer;
        int bufferSize, field_count, lineCount, maximum, newLineLength, point;
        int lineNumber = 1;
        int maximumLineLength = 0;
        int[] fieldLengthCount = {0, 0};
        string fileName;

        public bool reading()
        {
            bool b = false;
            Encoding MS932 = Encoding.GetEncoding(932);
            int n = 0;
            string line = EMPTY;

            try
            {
                StreamReader file = new StreamReader(fileName, MS932);

                for (; ; )
                {
                    line = file.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    n += 1;
                    if (n == point)
                    {
                        b = true;
                        break;
                    }

                }

                file.Close();

                if (measure)
                {
                    Console.Write(fileName);
                    Console.Write(": ");
                    Console.Write(lineCount);
                    if (n == 1)
                    {
                        Console.WriteLine(" line");
                    }
                    else
                    {
                        Console.WriteLine(" lines");
                    }

                    Console.Write("maximum LF-to-LF span of ");
                    Console.Write(maximumLineLength);
                    Console.Write(" bytes (line-");
                    Console.Write(lineNumber);
                    Console.WriteLine(")");
                }

                Console.WriteLine(line);
            }
            catch (IOException e39)
            {
                Console.Error.WriteLine(fileName);
                Error.handler(e39.ToString());
            }

            return b;
        }

        public bool reading(int separator)
        {
            bool b = false;
            Encoding MS932 = Encoding.GetEncoding(932);
            int fieldLength;
            int n = 0;
            string line = EMPTY;

            try
            {
                StreamReader file = new StreamReader(fileName, MS932);

                for (; ; )
                {
                    line = file.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    n += 1;
                    if (n == point)
                    {
                        b = true;
                        break;
                    }

                }

                file.Close();

                if (measure)
                {
                    Console.Write(fileName);
                    Console.Write(": ");
                    Console.Write(lineCount);
                    if (n == 1)
                    {
                        Console.WriteLine(" line");
                    }
                    else
                    {
                        Console.WriteLine(" lines");
                    }

                    buffer = MS932.GetBytes(line);
                    fieldLengthCount = maximumSpan(separator, line.Length );
                    fieldLength = fieldLengthCount[0] - SEPARATOR_SIZE;
                    field_count = fieldLengthCount[1];
                    Console.Write("maximum separator-to-separator span of ");
                    Console.Write(fieldLength);
                    Console.Write(" bytes (line-");
                    Console.Write(point);
                    Console.WriteLine(")");
                }

                Console.WriteLine(line);
            }
            catch (IOException e40)
            {
                Console.Error.WriteLine(fileName);
                Error.handler(e40.ToString());
            }

            return b;
        }

        public int fieldCount()
        {
            return field_count;
        }

        public int maximumSpan()
        {
            int c, m;
            int count = 0;
            int k;
            int max = 0;
            int trace = NOT_FOUND;

            carriageReturn = false;
            newLineLength = 1;
            for (k = 0; k < bufferSize && trace == NOT_FOUND; ++k)
            {
                c = (int)buffer[k];

                switch(c)
                {
                    case 0x0A:
                        count += 1;

                        if (carriageReturn)
                        {
                            max = k - 1 + newLineLength;
                        }
                        else
                        {
                            max = k + newLineLength;
                        }

                        lineNumber = count;
                        trace = k; // trace != NOT_FOUND
                        break;
                    case 0x0D:
                        carriageReturn = true;
                        newLineLength = 2;
                        break;
                    default:
                        break;
                }

            }

            for ( ; k < bufferSize; ++k)
            {
                c = (int)buffer[k];

                switch(c)
                {
                    case 0x0A:
                        count += 1;
                        m = k - trace;
                        if (m > max)
                        {
                            max = m;
                            lineNumber = count;
                        }

                        trace = k;
                        break;
                    default:
                        break;
                }

            }

            lineCount = count;
            return max;
        }

        public int[] maximumSpan(int separator, int size)
        {
            int c, m;
            int count = 0;
            int k;
            int max = 0;
            int trace = NOT_FOUND;
            int[] lengthCount = {0, 0};

            for (k = 0; k < size; ++k)
            {
                c = (int)buffer[k];

                if(c == separator)
                {
                    count += 1;
                    max = k + SEPARATOR_SIZE;
                    trace = k;
                    break;
                }

            }

            for( ; k < size; ++k)
            {
                c = (int)buffer[k];

                if(c == separator)
                {
                    count += 1;
                    m = k - trace;
                    if(m > max)
                    {
                        max = m;
                    }

                    trace = k;
                }

            }

            lengthCount[0] = max;
            lengthCount[1] = count;
            return lengthCount;
        }

        public int maximumLineSize()
        {
            return maximumLineLength;
        }

        public int textLineCount()
        {
            return lineCount;
        }

        public Line(string f)
        {
            fileName = f;
            measure = false;
            point = 1;
            bufferSize = (int)fileSize();
            if (bufferSize >= 0)
            {
                buffer = new byte[bufferSize];

                Stream binaryFile = File.OpenRead(fileName);
                binaryFile.Read(buffer, 0, bufferSize);
                maximum = maximumSpan(); // carriageReturn, lineCount, lineNumber
                newLineLength = carriageReturn ? 2 : 1;
                maximumLineLength = maximum - newLineLength;
                binaryFile.Close();
            }

        }

        public Line(string f, int p, bool m)
        {
            fileName = f;
            measure = m;
            point = p;
            bufferSize = (int)fileSize();
            if (bufferSize >= 0)
            {
                buffer = new byte[bufferSize];

                Stream binaryFile = File.OpenRead(fileName);
                binaryFile.Read(buffer, 0, bufferSize);
                maximum = maximumSpan(); // carriageReturn, lineCount, lineNumber
                newLineLength = carriageReturn ? 2 : 1;
                maximumLineLength = maximum - newLineLength;
                binaryFile.Close();
            }

        }

        public long fileSize()
        {
            long size = FILE_ERROR;

            try
            {
                FileStream f = new FileStream(fileName, FileMode.Open, FileAccess.Read);
                size = (long) f.Length;
                f.Close();
            }
            catch (IOException e41)
            {
                Console.Error.WriteLine(fileName);
                Error.handler(e41.ToString());
            }

            return size;
        }

        public void smallerBuffer(string line)
        {
            Encoding MS932 = Encoding.GetEncoding(932);
            int size = line.Length;

            if (bufferSize >= size)
            {
                bufferSize = size;
                buffer = MS932.GetBytes(line);
            }

        }

        public void smallerBuffer(string line, bool clear)
        {
            Encoding MS932 = Encoding.GetEncoding(932);
            int size = line.Length;

            if(bufferSize >= size)
            {
                if(clear)
                {
                    for (int i = 0; i < buffer.Length; ++i)
                    {
                        buffer[i] = 0;
                    }

                }

                bufferSize = size;
                buffer = MS932.GetBytes(line);
            }

        }

    }
}
