﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Rufeol
{
    class RemoveFromLine
    {
        const int BLOCK_SIZE = 1024;
        const int ERROR_CODE = 1;
        const int FULL_LEVEL = 1;
        const int LF_SIZE = 1;
        const long FILE_ERROR = -1L;
        const string CRLF = "\x0D\x0A";
        const string EMPTY = "";
        const string LF = "\x0A";
        const string TEMPORARY_BINARY = "binary.tmp";

        bool carriageReturn;
        byte[] buffer;
        byte[] os;
        int bufferSize, fullLevel, ring_count, size, unnecessary;
        long file_size;
        Queue<byte> ring = new Queue<byte>();
        byte BELL = 0x07;
        string fileName;

        delegate int NewLine(int k);

        int eucEOL(int k)
        {
            os[k++] = 0x0A;
            return k;
        }

        int sjisEOL(int k)
        {
            os[k++] = 0x0D;
            os[k++] = 0x0A;
            return k;
        }

        private bool renameBinaryFile(string binaryName)
        {
            bool b = true;
            byte[] c = new byte[BLOCK_SIZE];
            int size;

            try
            {
                File.Delete(binaryName);
                Stream editedFile = File.OpenRead(TEMPORARY_BINARY);
                Stream finalFile = File.OpenWrite(binaryName);

                for(; ; ) {
                    size = editedFile.Read(c, 0, BLOCK_SIZE);
                    if (size > 0)
                    {
                        finalFile.Write(c, 0, size);
                    }
                    else
                    {
                        break;
                    }

                }

                editedFile.Close();
                finalFile.Close();
                File.Delete(TEMPORARY_BINARY);
            }
            catch (IOException e50)
            {
                Console.Error.WriteLine(TEMPORARY_BINARY);
                Error.handler(e50.ToString());
                b = false;
            }

            return b;
        }

        private void prepareRing()
        {
            ring.Clear();
            for (int i = 0; i < ring_count; ++i)
            {
                ring.Enqueue(BELL);
            }

        }

        public long fileSize()
        {
            long size = FILE_ERROR;

            try
            {
                FileStream f = new FileStream(fileName, FileMode.Open, FileAccess.Read);
                size = (long) f.Length;
                f.Close();
            }
            catch (IOException e51)
            {
                Console.Error.WriteLine(fileName);
                Error.handler(e51.ToString());
            }

            return size;
        }

        public RemoveFromLine(string f, int l, int u, bool c)
        {
            carriageReturn = c;
            fileName = f;
            fullLevel = l;
            unnecessary = u;

            file_size = fileSize();
            if (file_size >= 0)
            {
                bufferSize = (int)file_size;
                buffer = new byte[bufferSize];
                os = new byte[bufferSize];
            }

        }

        public int reading(string newName)
        {
            bool full = false;
            byte c, s;
            int count = 0;
            int exit_code = 0;
            int k = 0;
            NewLine newLine = eucEOL;

            if (carriageReturn)
            {
                newLine = sjisEOL;
            }

// ring buffer
            ring_count = unnecessary + LF_SIZE;
            prepareRing();

            try
            {
                File.Delete(TEMPORARY_BINARY);
            }
            catch (IOException e76)
            {
                Error.handler(e76.ToString());
            }

            try
            {
                Stream binaryFile = File.OpenRead(fileName);
                Stream newFile = File.OpenWrite(TEMPORARY_BINARY);
                size = binaryFile.Read(buffer, 0, bufferSize);

                for (int i = 0; i < size; ++i)
                {
                    c = buffer[i];

                    if(c == 0x0A) {
                        count += 1;
                        if(count >= fullLevel) {
                            full = true;
                        }

                    }

                    ring.Dequeue();
                    ring.Enqueue(c);

                    if (full)
                    {
                        k = newLine(k);
                        prepareRing();
                        count = 0;
                        full = false;
                    }
                    else
                    {
                        s = ring.Peek();
                        if(s == BELL)
                        {
                            continue;
                        }
                        else
                        {
                            os[k++] = s;
                        }

                    }

                } // for(int i = 0; i < size; ++i)

                newFile.Write(os, 0, k);
                newFile.Close();
                binaryFile.Close();
                renameBinaryFile(newName);
            }
            catch (IOException e52)
            {
                Console.Error.WriteLine(fileName);
                Error.handler(e52.ToString());
                exit_code = ERROR_CODE;
            }

            return exit_code;
        }

    }
}
