﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;

namespace Sortk
{
    class Standard_input
    {   // UTF-8
        const bool NO_REMOVING = false;
        const char DAKUTEN = 'ﾞ';
        const char HANDAKUTEN = 'ﾟ';
        const int ERROR_CODE = 1;
        const int FIELD_COUNT = 3000;
        const int FLOATING_POINT = 2;
        const int INTEGER = 1;
        const int PHRASES = 0;
        const string EMPTY = "";
        const string NEW_LINE = "\x0D\x0A";

        bool combiningCharacter, omitFromASCII, skipSpace;
        char dakuten, handakuten, separator;
        Dividing d;
        Encoding encoded;
        int factor, fieldCount, from, length_1, to;
        string bits;

        private string supplyingBits(int fieldCount)
        {
            const char SET = '1';
            StringBuilder os = new StringBuilder();

            for (int i = 0; i < fieldCount; ++i)
            {
                os.Append(SET);
            }

            return os.ToString(); // 1111111111111...
        }

        public bool reading_double()
        {
            bool b = true;
            int q;
            List<DoubleContainer> fieldsListArray = new List<DoubleContainer>();
            string line;
            List<string> a;

            try
            {
                for (; ; )
                {
                    line = Console.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    length_1 = line.Length - 1;
                    if (length_1 >= 0)
                    {
                        q = line.LastIndexOf(separator);
                        if (q == length_1)
                        {
                            // Removing an extra separator
                            line = line.Substring(0, q);
                        }

                    }

                    d = new Dividing(line, bits, fieldCount, separator, NO_REMOVING);
                    d.smashed_to_pieces();
                    a = d.pieces();
                    fieldsListArray.Add( new DoubleContainer(a, factor, from, separator) );
                }

                fieldsListArray.Sort();
                Stream standard_output = Console.OpenStandardOutput();
                fieldsListArray.ForEach( f => writeBytes(standard_output, f.join() ) );
            }
            catch (IOException e71)
            {
                Error.handler(e71.ToString());
                b = false;
            }

            return b;
        }

        public bool reading_integer()
        {
            bool b = true;
            int q;
            List<IntegerContainer> fieldsListArray = new List<IntegerContainer>();
            string line;
            List<string> a;

            try
            {
                for (; ; )
                {
                    line = Console.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    length_1 = line.Length - 1;
                    if (length_1 >= 0)
                    {
                        q = line.LastIndexOf(separator);
                        if (q == length_1)
                        {
                            // Removing an extra separator
                            line = line.Substring(0, q);
                        }

                    }

                    d = new Dividing(line, bits, fieldCount, separator, NO_REMOVING);
                    d.smashed_to_pieces();
                    a = d.pieces();
                    fieldsListArray.Add( new IntegerContainer(a, factor, from, separator) );
                }

                fieldsListArray.Sort();
                Stream standard_output = Console.OpenStandardOutput();
                fieldsListArray.ForEach( f => writeBytes(standard_output, f.join() ) );
            }
            catch (IOException e72)
            {
                Error.handler(e72.ToString());
                b = false;
            }

            return b;
        }

        public bool reading_phrases()
        {
            bool b = true;
            int q;
            List<FieldsList> fieldsListArray = new List<FieldsList>();
            string line;
            List<string> a;

            try
            {
                for (; ; )
                {
                    line = Console.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    length_1 = line.Length - 1;
                    if (length_1 >= 0)
                    {
                        q = line.LastIndexOf(separator);
                        if (q == length_1)
                        {
                            // Removing an extra separator
                            line = line.Substring(0, q);
                        }

                    }

                    d = new Dividing(line, bits, fieldCount, separator, NO_REMOVING);
                    d.smashed_to_pieces();
                    a = d.pieces();
                    fieldsListArray.Add( new FieldsList(a, factor, from, to, dakuten, handakuten, separator, combiningCharacter, skipSpace) );
                }

                fieldsListArray.Sort();
                Stream standard_output = Console.OpenStandardOutput();
                fieldsListArray.ForEach( f => writeBytes(standard_output, f.join() ) );
            }
            catch (IOException e64)
            {
                Error.handler(e64.ToString());
                b = false;
            }

            return b;
        }

        public bool writeBytes(string line)
        {
            bool b = true;
            byte[] buffer;
            int size;

            try
            {
                Stream standard_output = Console.OpenStandardOutput();
                line += NEW_LINE;
                buffer = encoded.GetBytes(line);
                size = encoded.GetByteCount(line);
                standard_output.Write(buffer, 0, size);
            }
            catch (IOException e60)
            {
                Error.handler(e60.ToString());
                b = false;
            }

            return b;
        }

        public bool writeBytes(Stream newFile, string line)
        {
            bool b = true;
            byte[] buffer;
            int size;

            try
            {
                line += NEW_LINE;
                buffer = encoded.GetBytes(line);
                size = encoded.GetByteCount(line);
                newFile.Write(buffer, 0, size);
            }
            catch (IOException e62)
            {
                Error.handler(e62.ToString());
                b = false;
            }

            return b;
        }

        public int reading(int manager, bool ticktock)
        {
            bool b = true;

            if (ticktock)
            {
                Stopwatch w = Stopwatch.StartNew();

                switch (manager)
                {
                    case FLOATING_POINT:
                        b = reading_double();
                        break;
                    case INTEGER:
                        b = reading_integer();
                        break;
                    case PHRASES:
                        b = reading_phrases();
                        break;
                    default:
                        b = reading_phrases();
                        break;
                }

                w.Stop();
                Console.Write( w.ElapsedTicks );
                Console.WriteLine(" ticks");
            }
            else
            {
                switch (manager)
                {
                    case FLOATING_POINT:
                        b = reading_double();
                        break;
                    case INTEGER:
                        b = reading_integer();
                        break;
                    case PHRASES:
                        b = reading_phrases();
                        break;
                    default:
                        b = reading_phrases();
                        break;
                }

            }

            int exit_code = b ? 0 : ERROR_CODE;
            return exit_code;
        }

        public Standard_input(Encoding d, int r, int f, int t, char s, bool c, bool e, bool o)
        {
            Encoding EUC_JP = Encoding.GetEncoding("euc-jp");

            bits = supplyingBits(FIELD_COUNT);
            combiningCharacter = c;
            encoded = d;
            factor = r;
            fieldCount = FIELD_COUNT;
            from = f - 1;
            omitFromASCII = o;
            separator = s;
            skipSpace = e;
            to = t - 1;

            if (omitFromASCII)
            {
/*
Please see sortk.txt and check if DAKUTEN is substituted with H;
ﾞH
ﾟI
*/
                dakuten = 'H';
                handakuten = 'I';
            }
            else
            {
                dakuten = DAKUTEN;
                handakuten = HANDAKUTEN;
            }

        }

    }
}
